// Chris Chiar
// assignment two

import java.applet.*;
import java.awt.*;

// main class
public class shot extends Applet implements Runnable {

    // set object for threading
    Thread animation;

    // graphics image and offscreen buffer for double buffering
    Graphics offscreen;
    Image image;

    // how often things will happeng in the game
    static final int REFRESH_RATE = 80; // in ms

    // our angel and devil images
    Image angelImages[] = new Image[6]; // 2 angel Images
    Image devilImages[] = new Image[6]; // 2 devil Images
    // cloud images
    Image cloudsImage;

    // objects for managers of angeldevil and arrow
    ADManager       angeldevil_manager;
    ArrowManager    arrow_manager;

    // font for game over
    Font dorkfont =
        new Font("Courier", Font.BOLD+Font.ITALIC,36);

    // font for normal text
    Font normfont =
        new Font("Courier", Font.BOLD+Font.ITALIC,12);

    // set flag if game over
    boolean game_over = false;

    // some things to keep applet healthy and warm
    int width_applet;
    int height_applet;
    int score;
    int num_images = 5;
    int ik = 0;

    // intialize applet
    public void init()
    {
        // set the score
        score = 0;

        // deal with the dimensions
        width_applet = bounds().width;
        height_applet = bounds().height;

        // load the images
        loadImages();

        // instatiate the object for angeldevil manager
        angeldevil_manager =
        new ADManager(width_applet,height_applet,
                      angelImages,devilImages, this);

        // instantiate the object for arrow
        arrow_manager =
        new ArrowManager(25,(height_applet/2),0,(height_applet/2));

        // all double buffering set-ups
        image =
        createImage(width_applet,height_applet);    // make offscreen buffer
        offscreen = image.getGraphics();
        offscreen.setColor(Color.black);
    }


    // load the images
    public void loadImages()
    {

        // helps with the loading in of images
        MediaTracker t = new MediaTracker(this);

        // loads in the cloud images
        cloudsImage = getImage(getCodeBase(),
                           "clouds.gif");

        // adds the cloud images
        t.addImage(cloudsImage,0);

        // 5 angel images are loaded in
        for (int i=0;i < num_images; i++) {
            angelImages[i] = getImage(getCodeBase(),
                             "angel" + (i+1) + ".gif");
            t.addImage(angelImages[i],0);
        }

        // 5 devil images are loaded in
        for (int i=0;i < num_images; i++) {
            devilImages[i] = getImage(getCodeBase(),
                             "devil" + (i+1) + ".gif");
            t.addImage(devilImages[i],0);
        }

        // wait for all images to finish loading
        try {
            t.waitForAll();
        } catch (InterruptedException e) {

    }

        // check for errors
        if (t.isErrorAny()) {
            showStatus("Error Loading Images!");
        }
        else if (t.checkAll()) {
            showStatus("Images successfully loaded");
        }
    }

    // when things need to be updated
    public void updateManager()
    {
        angeldevil_manager.update();
    }

    // update which comes before paint
    public void update(Graphics g)
    {
        paint(g);
    }

    // the actual paint procedure
    public void paint(Graphics g)
    {

        ik++;
        if (ik == 250) { ik = 0;}

        // draw the rectangle
        offscreen.setColor(getBackground());
        offscreen.fillRect(0,0,width_applet,height_applet);
        offscreen.setColor(Color.black);

        // draw the clouds
        offscreen.drawImage(cloudsImage,ik,0,250-ik,250,this);
        offscreen.drawImage(cloudsImage,0,0,ik,250,this);

        // if not game over then keep score
        if (!game_over){
            offscreen.setFont(normfont);
            offscreen.setColor(Color.black);
            offscreen.drawString("score: " + score, 10, 10);

        }

        //if game over then display game over in big fat red letters
        if (game_over) {
            offscreen.setFont(normfont);
            offscreen.drawString("score: " + score, 10, 10);
            offscreen.setFont(dorkfont);
            offscreen.setColor(Color.red);
            offscreen.drawString("game over", 20, height_applet/2);
        }

        // do all the necessary buffer transfers
        angeldevil_manager.paint(offscreen);
        arrow_manager.paint(offscreen);
        g.drawImage(image,0,0,this);
    }


    // what to do if mouse button is pressed
    public boolean mouseDown(Event e, int x, int y)
    {
        // set the coordinated of tip of arrow
        arrow_manager.x_init = x;
        arrow_manager.y_init = y;

        // if arrow hits the devil
        if (angeldevil_manager.intersect_devil(x,y) && !game_over) {
            score++;
            angeldevil_manager.initializePosition(angeldevil_manager.angeldevil[1]);

            // reset arrow
            arrow_manager.x_init = 0;
	        arrow_manager.y_init = height_applet/2 + 25;
        }

        // if arrow hits the angel
        if (angeldevil_manager.intersect_angel(x,y)) {

            // reset arrow
            arrow_manager.x_init = 0;
	        arrow_manager.y_init = height_applet/2 + 25;

            game_over = true;
        }

        return true;
    }


    // moving the mouse around
    public boolean mouseMove(Event e, int x, int y)
    {

        // change y coordinate to midway of screen
        y = y - (height_applet/2);

        // calculate our theta for arrow rotation
        double theta = Math.atan(y/x);

        // do the necessary rotation calculations
        x = (int)(25*Math.cos(theta));
	    y = (int)(height_applet/2 + 25*Math.sin(theta));

        // the coordinates are set
        arrow_manager.x_init = x;
        arrow_manager.y_init = y;

        return true;
    }


    // start the thread
    public void start()
    {
        animation =  new Thread(this);
        if (animation != null) {
            animation.start();
        }
    }

    // run the thread
    public void run()
    {
        while (true) {
            repaint();
            updateManager();
            Thread.currentThread().yield();
            try {
                Thread.sleep (REFRESH_RATE);
            } catch (Exception e) {};
        }
    }


    // stop the tread
    public void stop()
    {
        if (animation != null) {
            animation.stop();
            animation = null;
        }
    }

}

// class to control angels and devils
class ADManager
{

    // some necessary data
    int width_applet;
    int height_applet;
    int num_images = 5;
    int num_sprites = 2;
    int width_sprite = 40;
    int height_sprite = 40;
    int type;

    // the actual angel devil sprite objects
    AngelDevil angeldevil[];

    // the class constructor
    public ADManager(int aw, int ah,
                     Image angelImages[],
                     Image devilImages[],
                     Applet a
                     ){
                        this.width_applet = aw;
                        this.height_applet = ah;

                        // the sprite is vectored
                        angeldevil = new AngelDevil[num_sprites];
                        for (int i=0; i < num_sprites; i++) {
                            type = i;
                            // and instantiated
                            angeldevil[i] =
                            new AngelDevil(aw,ah,angelImages,devilImages,type,a);
                            // set the position
                            initializePosition(angeldevil[i]);
                        }
                     }

    // if you hit an angel
    public boolean intersect_angel(int x, int y)
    {

          return ((angeldevil[0].locx < x) && (angeldevil[0].locx+33 > x)
               && (angeldevil[0].locy < y) && (angeldevil[0].locy+29 > y));

    }

    // if you hit a devil
    public boolean intersect_devil(int x, int y)
    {

          return ((angeldevil[1].locx < x) && (angeldevil[1].locx+33 > x)
               && (angeldevil[1].locy < y) && (angeldevil[1].locy+29 > y));

    }


    // setting the position
    public void initializePosition(AngelDevil ad)
    {

        // if it is a devil
        if (type == 1) {
            ad.setPosition(25+ad.getRand(width_applet-width_sprite-25),
            ad.getRand(height_applet-height_sprite));

            ad.setVelocity(12,12);
        }
        else
        {
            // or an angel
            ad.setPosition(25+ad.getRand(width_applet-width_sprite-25),
            ad.getRand(height_applet-height_sprite));

            ad.setVelocity(8,8);
        }
    }


    // paint the sprites
    public void paint(Graphics g)
    {
        for (int i=0; i < num_sprites; i++){
            angeldevil[i].paint(g);
        }
    }


    // update the sprites
    public void update()
    {
        for (int i=0; i < num_sprites; i++) {
            angeldevil[i].update();
        }
    }
}


// the actual class to move the angel and devil sprites
class AngelDevil {

    // necessary data
    int width_applet;
    int height_applet;
    int num_images = 5;
    int num_sprites = 2;
    int width_sprite = 33;
    int height_sprite = 29;
    int type;
    int currentImage = 1;
    protected int locx;
    protected int locy;
    protected int vx;
    protected int vy;


    // images to be used
    Image angelImages[];
    Image devilImages[];

    // the applet to deal with
    Applet a;

    // the constructor
    public AngelDevil (int aw, int ah,
                       Image angelImages[],
                       Image devilImages[],
                       int type,
                       Applet a)
                       {

                        this.width_applet = aw;
                        this.height_applet = ah;
                        this.angelImages = angelImages;
                        this.devilImages = devilImages;
                        this.type = type;
                        this.a = a;

                       }

    // update the sprite
    public void update ()
    {

    // update sprite state

    double r1 = Math.random();
    double r2 = Math.random();

    // change the velocity by flipping
    // the y-velocity. Net result:
    // sprite moves up and down. And
    // if ufo gets close to the top or
    // bottom of screen, it always changes
    // direction

    if ((locy < 0)  || (locy > height_applet-height_sprite)) {
        vy = -vy;
    }

    if ((locx < 25) || (locx > width_applet-width_sprite)) {
        vx = -vx;
    }

    // advance the image of the sprite
    currentImage = (currentImage + 1) % num_images;
    updatePosition();

    }

    // random generator
    static public int getRand(int x)
    {
        return (int)(x * Math.random());
    }

    // do the painting
    public void paint(Graphics g)
    {
        // devil sprite (type = 1)
        if (type == 1) {
            g.drawImage(devilImages[currentImage], locx, locy, a);
        }

        // angel sprite (type = 0)
        if (type == 0) {
            g.drawImage(angelImages[currentImage], locx, locy, a);
        }
    }


    // set the velocity
    public void setVelocity(int x, int y)
    {
        vx = x;
        vy = y;
    }

    // set the position
    public void setPosition(int x, int y)
    {
        locx = x;
        locy = y;
    }

    // move it around
    public void updatePosition()
    {
        locy += vy;
        locx += vx;
    }
}


// the class which moves the arrow
class ArrowManager
{

    // some data
    int x_head, y_head;
    int x_tip, y_tip;
    int x_init, y_init;
    int x_origin, y_origin;

    // and the rotation theta
    double theta;

    // and a nice little constructor
    public ArrowManager(int xinit, int yinit, int xorigin, int yorigin)
    {
        this.x_init = xinit;
        this.y_init = yinit;
        this.x_origin = xorigin;
        this.y_origin = yorigin;
    }

    // creating the tip
    public void tip_create()
    {
        x_tip = x_init;
        y_tip = y_init;
    }

    // rotate vector (x,y) by theta
    public void rotate(double theta)
    {
	    x_head = (int)(x_origin+(x_tip)*Math.cos(theta) - (y_tip)*Math.sin(theta));
	    y_head = (int)(y_origin+(x_tip)*Math.sin(theta) + (y_tip)*Math.cos(theta));
    }

    // paint that arrow
    public void paint(Graphics g)
    {

        // set the color
        g.setColor(Color.black);

	     // deal with body of arrow
	    tip_create();
	    g.drawLine(x_origin, y_origin, x_tip,y_tip);

    	// make tip of arrow relative to center of screen
	    x_tip = x_tip - x_origin;
	    y_tip = y_tip - y_origin;

	    // now reduce the size (vector) to find locations
	    // of the head flange points
        x_tip = (int) (x_tip * 0.75);
	    y_tip = (int) (y_tip * 0.75);

	    // rotate flange point to the right

	    rotate(0.35);

	    // draw the line for right flange
	    g.drawLine(x_init, y_init, x_head, y_head);

        // rotate flange point to the left

	    rotate(-0.35);

	    // draw the line for left flange
	    g.drawLine(x_init, y_init, x_head, y_head);
    }

}

// all she wrote





